@@ -203,7 +203,7 @@ gst_bin_get_type (void)
 {
   static GType gst_bin_type = 0;
 
-  if (!gst_bin_type) {
+  if (G_UNLIKELY (gst_bin_type == 0)) {
     static const GTypeInfo bin_info = {
       sizeof (GstBinClass),
       gst_bin_base_init,
@@ -348,7 +348,7 @@ gst_bin_class_init (GstBinClass * klass)
   GST_DEBUG ("creating bin thread pool");
   err = NULL;
   klass->pool =
-      g_thread_pool_new ((GFunc) gst_bin_recalc_func, NULL, 1, FALSE, &err);
+      g_thread_pool_new ((GFunc) gst_bin_recalc_func, NULL, -1, FALSE, &err);
   if (err != NULL) {
     g_critical ("could alloc threadpool %s", err->message);
   }
@@ -574,7 +574,7 @@ bin_replace_message (GstBin * bin, GstMessage * message, GstMessageType types)
   gboolean res = TRUE;
   const gchar *name;
 
-  name = gst_message_type_get_name (GST_MESSAGE_TYPE (message));
+  name = GST_MESSAGE_TYPE_NAME (message);
 
   if ((src = GST_MESSAGE_SRC (message))) {
     MessageFind find;
@@ -1962,7 +1962,7 @@ static void
 gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
 {
   GST_DEBUG_OBJECT (bin, "[msg %p] handling child message of type %s",
-      message, gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
+      message, GST_MESSAGE_TYPE_NAME (message));
 
   switch (GST_MESSAGE_TYPE (message)) {
     case GST_MESSAGE_EOS:
@@ -2005,8 +2005,13 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
 
       klass = GST_BIN_GET_CLASS (bin);
       gst_object_ref (bin);
-      GST_DEBUG_OBJECT (bin, "pushing recalc on thread pool");
-      g_thread_pool_push (klass->pool, bin, NULL);
+      if (!bin->polling) {
+        GST_DEBUG_OBJECT (bin, "pushing recalc on thread pool");
+        g_thread_pool_push (klass->pool, bin, NULL);
+      } else {
+        GST_DEBUG_OBJECT (bin,
+            "state recalc already in progress, not pushing new recalc");
+      }
       GST_OBJECT_UNLOCK (bin);
       break;
 
@@ -2015,12 +2020,8 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
     not_toplevel:
       {
         GST_OBJECT_UNLOCK (bin);
-        GST_DEBUG_OBJECT (bin, "not toplevel");
-
-        /* post message up, mark parent bins dirty */
-        gst_element_post_message (GST_ELEMENT_CAST (bin), message);
-
-        break;
+        GST_DEBUG_OBJECT (bin, "not toplevel, forwarding");
+        goto forward;
       }
     }
     case GST_MESSAGE_SEGMENT_START:
@@ -2098,12 +2099,11 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
           provided, playing, forward);
       GST_OBJECT_UNLOCK (bin);
 
-      if (forward) {
+      if (forward)
         goto forward;
-      }
+
       /* free message */
       gst_message_unref (message);
-
       break;
     }
     case GST_MESSAGE_CLOCK_PROVIDE:
@@ -2119,15 +2119,14 @@ gst_bin_handle_message_func (GstBin * bin, GstMessage * message)
 
       if (forward)
         goto forward;
+
       /* free message */
       gst_message_unref (message);
-
       break;
     }
     default:
       goto forward;
   }
-
   return;
 
 forward:
