@@ -142,7 +142,7 @@ gst_element_get_type (void)
 {
   static GType gst_element_type = 0;
 
-  if (!gst_element_type) {
+  if (G_UNLIKELY (gst_element_type == 0)) {
     static const GTypeInfo element_info = {
       sizeof (GstElementClass),
       gst_element_base_class_init,
@@ -1577,6 +1577,8 @@ gst_element_set_locked_state (GstElement * element, gboolean locked_state)
   return TRUE;
 
 was_ok:
+  GST_CAT_DEBUG (GST_CAT_STATES, "elements %s was in locked state %d",
+      GST_ELEMENT_NAME (element), old);
   GST_OBJECT_UNLOCK (element);
 
   return FALSE;
