@@ -25,7 +25,7 @@
 #define __GST_PLUGIN_FEATURE_H__
 
 #include <glib-object.h>
-#include <gst/gsttypes.h>
+#include <gst/gstobject.h>
 
 G_BEGIN_DECLS
 
@@ -36,45 +36,96 @@ G_BEGIN_DECLS
 #define GST_IS_PLUGIN_FEATURE_CLASS(klass) 	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PLUGIN_FEATURE))
 #define GST_PLUGIN_FEATURE_GET_CLASS(obj) 	(G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_PLUGIN_FEATURE, GstPluginFeatureClass))
 
+/**
+ * GST_PLUGIN_FEATURE_NAME:
+ * @feature: The feature to query
+ *
+ * Get the name of the feature
+ */
 #define GST_PLUGIN_FEATURE_NAME(feature)  (GST_PLUGIN_FEATURE (feature)->name)
 
 typedef struct _GstPluginFeature GstPluginFeature;
 typedef struct _GstPluginFeatureClass GstPluginFeatureClass;
 
+/**
+ * GstRank:
+ * @GST_RANK_NONE: will be chosen last or not at all
+ * @GST_RANK_MARGINAL: unlikly to be chosen
+ * @GST_RANK_SECONDARY: likely to be chosen
+ * @GST_RANK_PRIMARY: will be chosen first
+ *
+ * Element priority ranks. Defines the order in which the autoplugger (or
+ * similar rank-picking mechanisms) will choose this element over an alternative
+ * one with the same function.
+ *
+ * The rank is a unsigned integer ranging from 0 (GST_RANK_NONE) to 256
+ * (GST_RANK_PRIMARY). These constants serve as a rough guidiance for defining
+ * the rank of a #GstPlugin using gst_plugin_feature_set_rank().
+ */
+typedef enum {
+  GST_RANK_NONE                 = 0,
+  GST_RANK_MARGINAL             = 64,
+  GST_RANK_SECONDARY            = 128,
+  GST_RANK_PRIMARY              = 256
+} GstRank;
+
+/**
+ * GstPluginFeature:
+ *
+ * Opaque #GstPluginFeature structure.
+ */
 struct _GstPluginFeature {
-  GObject 	 object;
+  GstObject 	 object;
 
   /*< private >*/
+  gboolean       loaded;
   gchar 	*name;
   guint   	 rank;
 
-  gpointer 	 manager;
+  gchar *plugin_name;
 
+  /*< private >*/
   gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstPluginFeatureClass {
-  GObjectClass	parent_class;
-
-  void          (*unload_thyself)      (GstPluginFeature *feature);
+  GstObjectClass	parent_class;
 
+  /*< private >*/
   gpointer _gst_reserved[GST_PADDING];
 };
 
+/**
+ * GstTypeNameData:
+ * @name: a name
+ * @type: a GType
+ *
+ * Structure used for filtering based on @name and @type.
+ */
 typedef struct {
   const gchar 	*name;
   GType		 type;
 } GstTypeNameData;
 
-/* filter */
+/**
+ * GstPluginFeatureFilter:
+ * @feature: the pluginfeature to check
+ * @user_data: the user_data that has been passed on e.g.
+ *  gst_registry_feature_filter()
+ *
+ * A function that can be used with e.g. gst_registry_feature_filter()
+ * to get a list of pluginfeature that match certain criteria.
+ *
+ * Returns: %TRUE for a positive match, %FALSE otherwise
+ */
 typedef gboolean        (*GstPluginFeatureFilter)       (GstPluginFeature *feature,
                                                          gpointer user_data);
 
 /* normal GObject stuff */
 GType		gst_plugin_feature_get_type		(void);
 
-gboolean	gst_plugin_feature_ensure_loaded 	(GstPluginFeature *feature);
-void		gst_plugin_feature_unload_thyself 	(GstPluginFeature *feature);
+GstPluginFeature *
+                gst_plugin_feature_load         	(GstPluginFeature *feature);
 
 gboolean	gst_plugin_feature_type_name_filter	(GstPluginFeature *feature,
 							 GstTypeNameData *data);
@@ -84,6 +135,13 @@ void		gst_plugin_feature_set_name		(GstPluginFeature *feature, const gchar *name
 guint		gst_plugin_feature_get_rank		(GstPluginFeature *feature);
 G_CONST_RETURN gchar *gst_plugin_feature_get_name	(GstPluginFeature *feature);
 
+void            gst_plugin_feature_list_free            (GList *list);
+
+gboolean        gst_plugin_feature_check_version        (GstPluginFeature *feature,
+                                                         guint             min_major,
+                                                         guint             min_minor,
+                                                         guint             min_micro);
+
 G_END_DECLS
 
 
