@@ -82,6 +82,11 @@ GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
+/* Require at least 2kB of data before we attempt typefinding in chain-mode.
+ * 128kB is massive overkill for the maximum, but doesn't do any harm */
+#define TYPE_FIND_MIN_SIZE   (2*1024)
+#define TYPE_FIND_MAX_SIZE (128*1024)
+
 /* TypeFind signals and args */
 enum
 {
@@ -138,6 +143,8 @@ gst_type_find_element_change_state (GstElement * element,
 static gboolean gst_type_find_element_activate (GstPad * pad);
 static gboolean
 gst_type_find_element_activate_src_pull (GstPad * pad, gboolean active);
+static GstFlowReturn
+gst_type_find_element_chain_do_typefinding (GstTypeFindElement * typefind);
 
 static guint gst_type_find_element_signals[LAST_SIGNAL] = { 0 };
 
@@ -187,10 +194,11 @@ gst_type_find_element_class_init (GstTypeFindElementClass * typefind_class)
       g_param_spec_uint ("minimum", _("minimum"),
           "minimum probability required to accept caps", GST_TYPE_FIND_MINIMUM,
           GST_TYPE_FIND_MAXIMUM, GST_TYPE_FIND_MINIMUM, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, ARG_MINIMUM,
+  g_object_class_install_property (gobject_class, ARG_MAXIMUM,
       g_param_spec_uint ("maximum", _("maximum"),
-          "probability to stop typefinding", GST_TYPE_FIND_MINIMUM,
-          GST_TYPE_FIND_MAXIMUM, GST_TYPE_FIND_MAXIMUM, G_PARAM_READWRITE));
+          "probability to stop typefinding (deprecated; non-functional)",
+          GST_TYPE_FIND_MINIMUM, GST_TYPE_FIND_MAXIMUM, GST_TYPE_FIND_MAXIMUM,
+          G_PARAM_READWRITE));
   /**
    * GstTypeFindElement::have-type:
    * @typefind: the typefind instance
@@ -268,18 +276,14 @@ gst_type_find_element_set_property (GObject * object, guint prop_id,
 {
   GstTypeFindElement *typefind;
 
-  g_return_if_fail (GST_IS_TYPE_FIND_ELEMENT (object));
-
   typefind = GST_TYPE_FIND_ELEMENT (object);
 
   switch (prop_id) {
     case ARG_MINIMUM:
       typefind->min_probability = g_value_get_uint (value);
-      g_object_notify (object, "minimum");
       break;
     case ARG_MAXIMUM:
       typefind->max_probability = g_value_get_uint (value);
-      g_object_notify (object, "maximum");
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -292,8 +296,6 @@ gst_type_find_element_get_property (GObject * object, guint prop_id,
 {
   GstTypeFindElement *typefind;
 
-  g_return_if_fail (GST_IS_TYPE_FIND_ELEMENT (object));
-
   typefind = GST_TYPE_FIND_ELEMENT (object);
 
   switch (prop_id) {
@@ -389,42 +391,16 @@ gst_type_find_element_src_event (GstPad * pad, GstEvent * event)
   }
   return gst_pad_event_default (pad, event);
 }
-typedef struct
-{
-  GstTypeFindFactory *factory;
-  guint probability;
-  GstCaps *caps;
-  guint requested_size;
-  GstTypeFindElement *self;
-}
-TypeFindEntry;
 
-static inline TypeFindEntry *
-new_entry (void)
-{
-  return g_new0 (TypeFindEntry, 1);
-}
-static void
-free_entry (TypeFindEntry * entry)
-{
-  if (entry->caps)
-    gst_caps_unref (entry->caps);
-  gst_object_unref (entry->factory);
-  g_free (entry);
-}
 static void
 start_typefinding (GstTypeFindElement * typefind)
 {
-  g_assert (typefind->possibilities == NULL);
-
   GST_DEBUG_OBJECT (typefind, "starting typefinding");
   gst_pad_set_caps (typefind->src, NULL);
   if (typefind->caps) {
     gst_caps_replace (&typefind->caps, NULL);
   }
   typefind->mode = MODE_TYPEFIND;
-  typefind->stream_length_available = TRUE;
-  typefind->stream_length = 0;
 }
 
 static void
@@ -433,22 +409,12 @@ stop_typefinding (GstTypeFindElement * typefind)
   GstState state;
   gboolean push_cached_buffers;
 
-  gst_element_get_state (GST_ELEMENT (typefind), &state, NULL,
-      GST_CLOCK_TIME_NONE);
+  gst_element_get_state (GST_ELEMENT (typefind), &state, NULL, 0);
 
   push_cached_buffers = (state >= GST_STATE_PAUSED);
 
   GST_DEBUG_OBJECT (typefind, "stopping typefinding%s",
       push_cached_buffers ? " and pushing cached buffers" : "");
-  if (typefind->possibilities != NULL) {
-    /* this should only happen on PAUSED => READY or EOS */
-    GST_LOG_OBJECT (typefind, "freeing remaining %u typefind functions",
-        g_list_length (typefind->possibilities));
-    g_list_foreach (typefind->possibilities, (GFunc) free_entry, NULL);
-    g_list_free (typefind->possibilities);
-    typefind->possibilities = NULL;
-  }
-  //typefind->mode = MODE_TRANSITION;
 
   if (typefind->store) {
     if (!push_cached_buffers) {
@@ -484,64 +450,6 @@ stop_typefinding (GstTypeFindElement * typefind)
   }
 }
 
-static guint64
-find_element_get_length (gpointer data)
-{
-  TypeFindEntry *entry = (TypeFindEntry *) data;
-  GstTypeFindElement *typefind = entry->self;
-  GstFormat format = GST_FORMAT_BYTES;
-
-  if (!typefind->stream_length_available) {
-    GST_LOG_OBJECT (entry->self,
-        "'%s' called get_length () but we know it's not available",
-        GST_PLUGIN_FEATURE_NAME (entry->factory));
-    return 0;
-  }
-  if (entry->self->stream_length == 0) {
-    if (!gst_pad_query_duration (GST_PAD_PEER (entry->self->sink), &format,
-            (gint64 *) & entry->self->stream_length))
-      goto no_length;
-
-    if (format != GST_FORMAT_BYTES) {
-      typefind->stream_length_available = FALSE;
-      entry->self->stream_length = 0;
-    } else {
-      GST_DEBUG_OBJECT (entry->self,
-          "'%s' called get_length () and it's %" G_GUINT64_FORMAT " bytes",
-          GST_PLUGIN_FEATURE_NAME (entry->factory), entry->self->stream_length);
-    }
-  }
-  return entry->self->stream_length;
-
-no_length:
-  {
-    typefind->stream_length_available = FALSE;
-    GST_DEBUG_OBJECT (entry->self,
-        "'%s' called get_length () but it's not available",
-        GST_PLUGIN_FEATURE_NAME (entry->factory));
-    return 0;
-  }
-}
-
-static TypeFindEntry *
-gst_type_find_element_get_best_possibility (GstTypeFindElement * typefind)
-{
-  TypeFindEntry *best = NULL;
-  GList *walk;
-
-  for (walk = typefind->possibilities; walk != NULL; walk = walk->next) {
-    TypeFindEntry *entry;
-
-    entry = (TypeFindEntry *) walk->data;
-    if ((!best || entry->probability > best->probability) &&
-        entry->probability >= typefind->min_probability) {
-      best = entry;
-    }
-  }
-
-  return best;
-}
-
 static gboolean
 gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
 {
@@ -555,20 +463,24 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
     case MODE_TYPEFIND:
       switch (GST_EVENT_TYPE (event)) {
         case GST_EVENT_EOS:{
-          TypeFindEntry *entry;
+          GstTypeFindProbability prob;
+          GstCaps *caps;
 
-          entry = gst_type_find_element_get_best_possibility (typefind);
+          GST_INFO_OBJECT (typefind, "Got EOS and no type found yet");
 
-          if (entry && entry->probability >= typefind->min_probability) {
-            GST_INFO_OBJECT (typefind,
-                "'%s' is the best typefind left after we got all data, using it now (probability %u)",
-                GST_PLUGIN_FEATURE_NAME (entry->factory), entry->probability);
+          /* we might not have started typefinding yet because there was not
+           * enough data so far; just give it a shot now and see what we get */
+          caps = gst_type_find_helper_for_buffer (GST_OBJECT (typefind),
+              typefind->store, &prob);
+
+          if (caps && prob >= typefind->min_probability) {
             g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
-                0, entry->probability, entry->caps);
+                0, prob, caps);
           } else {
-            GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL),
-                (NULL));
+            GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND,
+                (NULL), (NULL));
           }
+          gst_caps_replace (&caps, NULL);
           stop_typefinding (typefind);
           res = gst_pad_event_default (pad, event);
           break;
@@ -595,71 +507,12 @@ gst_type_find_element_handle_event (GstPad * pad, GstEvent * event)
   }
   return res;
 }
-static guint8 *
-find_peek (gpointer data, gint64 offset, guint size)
-{
-  TypeFindEntry *entry = (TypeFindEntry *) data;
-
-  GST_LOG_OBJECT (entry->self, "'%s' called peek (%" G_GINT64_FORMAT ", %u)",
-      GST_PLUGIN_FEATURE_NAME (entry->factory), offset, size);
-  if (offset != 0)
-    return NULL;
-
-  if (size <= entry->self->store->size) {
-    entry->requested_size = 0;
-    return GST_BUFFER_DATA (entry->self->store);
-  } else {
-    entry->requested_size = size;
-
-    GST_LOG_OBJECT (entry->self,
-        "setting requested peek (%" G_GINT64_FORMAT ", %u) on '%s'", offset,
-        size, GST_PLUGIN_FEATURE_NAME (entry->factory));
-    return NULL;
-  }
-}
-static void
-find_suggest (gpointer data, guint probability, const GstCaps * caps)
-{
-  TypeFindEntry *entry = (TypeFindEntry *) data;
-
-  GST_LOG_OBJECT (entry->self, "'%s' called suggest (%u, %" GST_PTR_FORMAT ")",
-      GST_PLUGIN_FEATURE_NAME (entry->factory), probability, caps);
-  if (((gint) probability) > entry->probability) {
-    entry->probability = probability;
-    gst_caps_replace (&entry->caps, (GstCaps *) caps);
-  }
-}
-
-#if 0
-static gint
-compare_type_find_entry (gconstpointer a, gconstpointer b)
-{
-  TypeFindEntry *one = (TypeFindEntry *) a;
-  TypeFindEntry *two = (TypeFindEntry *) b;
-
-  if (one->probability == two->probability) {
-    /* FIXME: can be improved by analyzing requests */
-    return 0;
-  } else {
-    return two->probability - one->probability;
-  }
-}
-#endif
-
-static gint
-compare_type_find_factory (gconstpointer fac1, gconstpointer fac2)
-{
-  return GST_PLUGIN_FEATURE (fac1)->rank - GST_PLUGIN_FEATURE (fac2)->rank;
-}
 
 static GstFlowReturn
 gst_type_find_element_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstTypeFindElement *typefind;
-  TypeFindEntry *entry;
-  GList *walk;
   GstFlowReturn res = GST_FLOW_OK;
-  GstTypeFind find = { find_peek, find_suggest, NULL, find_element_get_length };
 
   typefind = GST_TYPE_FIND_ELEMENT (GST_PAD_PARENT (pad));
 
@@ -671,110 +524,12 @@ gst_type_find_element_chain (GstPad * pad, GstBuffer * buffer)
       gst_buffer_set_caps (buffer, typefind->caps);
       return gst_pad_push (typefind->src, buffer);
     case MODE_TYPEFIND:{
-      gboolean done = TRUE;
-
       if (typefind->store)
         typefind->store = gst_buffer_join (typefind->store, buffer);
       else
         typefind->store = buffer;
 
-      if (typefind->possibilities == NULL) {
-        /* not yet started, get all typefinding functions into our "queue" */
-        GList *all_factories = gst_type_find_factory_get_list ();
-
-        GST_INFO_OBJECT (typefind, "starting with %u typefinding functions",
-            g_list_length (all_factories));
-
-        all_factories = g_list_sort (all_factories, compare_type_find_factory);
-        walk = all_factories;
-        while (walk) {
-          entry = new_entry ();
-
-          entry->factory = gst_object_ref (GST_TYPE_FIND_FACTORY (walk->data));
-          entry->self = typefind;
-          entry->probability = 0;
-          typefind->possibilities =
-              g_list_prepend (typefind->possibilities, entry);
-          walk = g_list_next (walk);
-        }
-        gst_plugin_feature_list_free (all_factories);
-      }
-
-      /* call every typefind function once */
-      walk = typefind->possibilities;
-      GST_INFO_OBJECT (typefind, "iterating %u typefinding functions",
-          g_list_length (walk));
-      while (walk) {
-        find.data = entry = (TypeFindEntry *) walk->data;
-        if (entry->probability != 0) {
-          /* Probability already known, just continue along the list */
-          walk = g_list_next (walk);
-          continue;
-        }
-
-        entry->requested_size = 0;
-        gst_type_find_factory_call_function (entry->factory, &find);
-
-        if (entry->probability == 0 && entry->requested_size == 0) {
-          GList *next;
-
-          GST_DEBUG_OBJECT (typefind,
-              "'%s' was removed - no chance of being the right plugin",
-              GST_PLUGIN_FEATURE_NAME (entry->factory));
-          next = g_list_next (walk);
-          free_entry (entry);
-          typefind->possibilities =
-              g_list_delete_link (typefind->possibilities, walk);
-          walk = next;
-        } else if (entry->probability >= typefind->max_probability) {
-          /* wooha, got caps */
-          GstCaps *found_caps = entry->caps;
-          guint probability = entry->probability;
-
-          GST_INFO_OBJECT (typefind,
-              "'%s' returned %u/%u probability, using it NOW",
-              GST_PLUGIN_FEATURE_NAME (entry->factory), probability,
-              typefind->max_probability);
-          g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE], 0,
-              probability, found_caps);
-
-          g_list_foreach (typefind->possibilities, (GFunc) free_entry, NULL);
-          g_list_free (typefind->possibilities);
-          typefind->possibilities = NULL;
-          break;
-        } else {
-          walk = g_list_next (walk);
-          if (entry->requested_size != 0)
-            done = FALSE;
-        }
-      }
-
-      /* we may now already have caps or we might be left without functions to try */
-      if (typefind->caps) {
-        stop_typefinding (typefind);
-      } else if (typefind->possibilities == NULL) {
-        GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
-        stop_typefinding (typefind);
-        return GST_FLOW_ERROR;
-      } else if (done) {
-        TypeFindEntry *best;
-
-        if ((best = gst_type_find_element_get_best_possibility (typefind))) {
-          GST_INFO_OBJECT (typefind,
-              "'%s' is the best typefind left, using it now (probability %u)",
-              GST_PLUGIN_FEATURE_NAME (best->factory), best->probability);
-          g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
-              0, best->probability, best->caps);
-          g_list_foreach (typefind->possibilities, (GFunc) free_entry, NULL);
-          g_list_free (typefind->possibilities);
-          typefind->possibilities = NULL;
-          stop_typefinding (typefind);
-        } else {
-          GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
-          stop_typefinding (typefind);
-          return GST_FLOW_ERROR;
-        }
-      }
+      res = gst_type_find_element_chain_do_typefinding (typefind);
 
       if (typefind->mode == MODE_ERROR)
         res = GST_FLOW_ERROR;
@@ -789,6 +544,56 @@ gst_type_find_element_chain (GstPad * pad, GstBuffer * buffer)
   return res;
 }
 
+static GstFlowReturn
+gst_type_find_element_chain_do_typefinding (GstTypeFindElement * typefind)
+{
+  GstTypeFindProbability probability;
+  GstCaps *caps;
+
+  if (GST_BUFFER_SIZE (typefind->store) < TYPE_FIND_MIN_SIZE) {
+    GST_DEBUG_OBJECT (typefind, "not enough data for typefinding yet "
+        "(%u bytes)", GST_BUFFER_SIZE (typefind->store));
+    return GST_FLOW_OK;
+  }
+
+  caps = gst_type_find_helper_for_buffer (GST_OBJECT (typefind),
+      typefind->store, &probability);
+
+  if (caps == NULL && GST_BUFFER_SIZE (typefind->store) > TYPE_FIND_MAX_SIZE) {
+    GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+    stop_typefinding (typefind);
+    return GST_FLOW_ERROR;
+  } else if (caps == NULL) {
+    GST_DEBUG_OBJECT (typefind, "no caps found with %u bytes of data, "
+        "waiting for more data", GST_BUFFER_SIZE (typefind->store));
+    return GST_FLOW_OK;
+  }
+
+  /* found a type */
+  if (probability < typefind->min_probability) {
+    GST_DEBUG_OBJECT (typefind, "found caps %" GST_PTR_FORMAT ", but "
+        "probability is %u which is lower than the required minimum of %u",
+        caps, probability, typefind->min_probability);
+
+    if (GST_BUFFER_SIZE (typefind->store) >= TYPE_FIND_MAX_SIZE) {
+      GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+      stop_typefinding (typefind);
+      return GST_FLOW_ERROR;
+    }
+
+    GST_DEBUG_OBJECT (typefind, "waiting for more data to try again");
+    return GST_FLOW_OK;
+  }
+
+  /* probability is good enough too, so let's make it known ... */
+  g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE], 0,
+      probability, caps);
+
+  /* .. and send out the accumulated data */
+  stop_typefinding (typefind);
+  return GST_FLOW_OK;
+}
+
 static gboolean
 gst_type_find_element_checkgetrange (GstPad * srcpad)
 {
@@ -829,6 +634,7 @@ gst_type_find_element_activate_src_pull (GstPad * pad, gboolean active)
 static gboolean
 gst_type_find_element_activate (GstPad * pad)
 {
+  GstTypeFindProbability probability = 0;
   GstCaps *found_caps = NULL;
   GstTypeFindElement *typefind;
 
@@ -845,7 +651,7 @@ gst_type_find_element_activate (GstPad * pad)
    */
 
   /* 1 */
-  if (!gst_pad_activate_pull (pad, TRUE)) {
+  if (!gst_pad_check_pull_range (pad) || !gst_pad_activate_pull (pad, TRUE)) {
     start_typefinding (typefind);
     return gst_pad_activate_push (pad, TRUE);
   }
@@ -859,8 +665,15 @@ gst_type_find_element_activate (GstPad * pad)
       gint64 size;
       GstFormat format = GST_FORMAT_BYTES;
 
-      gst_pad_query_duration (peer, &format, &size);
-      found_caps = gst_type_find_helper (peer, (guint64) size);
+      if (!gst_pad_query_duration (peer, &format, &size)) {
+        GST_WARNING_OBJECT (typefind, "Could not query upstream length!");
+        return FALSE;
+      }
+
+      found_caps = gst_type_find_helper_get_range (GST_OBJECT (peer),
+          (GstTypeFindHelperGetRangeFunction) (GST_PAD_GETRANGEFUNC (peer)),
+          (guint64) size, &probability);
+
       gst_object_unref (peer);
     }
   }
@@ -872,14 +685,15 @@ gst_type_find_element_activate (GstPad * pad)
   gst_pad_activate_push (typefind->src, FALSE);
 
   /* 5 */
-  if (!found_caps) {
+  if (!found_caps || probability < typefind->min_probability) {
     GST_ELEMENT_ERROR (typefind, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+    gst_caps_replace (&found_caps, NULL);
     return FALSE;
   }
 
   /* 6 */
   g_signal_emit (typefind, gst_type_find_element_signals[HAVE_TYPE],
-      0, 100, found_caps);
+      0, probability, found_caps);
   gst_caps_unref (found_caps);
   typefind->mode = MODE_NORMAL;
 
